;nyquist plug-in
;version 1
;type process
;categories "http://lv2plug.in/ns/lv2core/#AmplifierPlugin"
;name "Target Period Fade..."
;action "Applying fade effect..."
;info "by Steve Daulton (http://easyspacepro.com) GPL v2.\nApplies a linear fade such that the peak level in the first (duration) seconds\nand the final (duration) seconds meet the target levels.\n\nLevels below -96 dB produce silence."

;;'Simple Target Fade' by Steve Daulton
;;Copyright (C) 2010 Steve Daulton
;;Licensed und terms of GNU General Public License, version 2.
;;http://www.gnu.org/licenses/gpl-2.0.html

;control itarg "Initial Target Level" real "dB" -6 -100 0
;control iwinsize "For First (duration) Seconds" real "seconds" 1 0 100
;control ftarg "Final Target Level" real "dB" -6 -100 0
;control fwinsize "For Last (duration) Seconds" real "seconds" 1 0 100

#|
Effect calculates the peak level for set period at the start and
end of the selection.
The selection is then amplified such that the peak at the start/end of
the selection are amplified to the initial and final levels.
If the total length of the selection is less than the sum of the start
and end periods an error message is shown.
If the peak level during the start/end duration is zero, gain is set to
unity.
|#

;;Convert target level to linear range 0 to 1
(defun constrain (val)
  (if (< val -96) 0
    (if (> val 0) 1 (db-to-linear val))))

(setq itarg (constrain itarg)) ; initial target
(setq ftarg (constrain ftarg)) ; final target

;; test for valid input
(setq err "") ; initialise error message to null string
(if (or (< iwinsize 0)(< fwinsize 0))
  (setq err "durations cannot be negative."))
(if (> (+ iwinsize fwinsize)(get-duration 1))
  (setq err "durations exceed length of selection."))

;; minimum window size = 1 sample
(if (= iwinsize 0)(setq iwinsize (/ *sound-srate*)))
(if (= fwinsize 0)(setq fwinsize (/ *sound-srate*)))

(defun fade-env (s-in isection fsection)
  (setq ipeak (peak (extract-abs 0 isection (cue s-in)) ny:all))
  (setq fpeak (peak (extract-abs (- (get-duration 1)fsection)(get-duration 1)(cue s-in)) ny:all))
  (setq iamp (if (> ipeak 0)(/ itarg ipeak) 1))
  (setq famp (if (> fpeak 0)(/ ftarg fpeak) 1))
  (pwl 0 iamp 1 famp))

(if (> (length err) 0)(format NIL "Error\n~a" err)
  (control-srate-abs *sound-srate*
    (mult s (fade-env s iwinsize fwinsize))))
